!These Classes are used Exclusively for the School Bookkeeper.
!They consist of the following files:
!  sbkCLASS.INC
!  sbkCLASS.CLW
!
!Errors are thrown regardless of whether the FileManager wants them to be
!   thrown.  The reason for this is that these errors should NEVER occur.

  MEMBER

    INCLUDE('sbkCLASS.INC'),ONCE
    INCLUDE('ABERROR.INC'),ONCE

HST                  FILE,DRIVER('TOPSPEED'),RECLAIM,PRE(HST),CREATE,BINDABLE,THREAD
k_Date                 KEY(HST:FileName,HST:TheDate,HST:TheTime),DUP,NOCASE
Buffer                 BLOB
Record                 RECORD,PRE()
Action                   BYTE
FileName                 STRING(FILE:MaxFilePath)
FileVersion              ULONG
Position                 STRING(4)
TheDate                  ULONG
TheTime                  ULONG
TheUser                  STRING(5)
Generic                  STRING(20)
                       END
                     END

ACTION               ITEMIZE,PRE()
Insert                 EQUATE
Change                 EQUATE
Delete                 EQUATE
                     END

HSTlogged            BYTE(False)
HSTopened            BYTE(False)
LoggedWithCB         BYTE(False)

EC                   &ErrorClass
FM                   &sbkFileManager

FileName             STRING(FILE:MaxFilePath)
ThePosition          STRING(4),AUTO
TheRecord            &GROUP

    MAP
AddHST               PROCEDURE(BYTE Action),BYTE,PRIVATE
CloseHST             PROCEDURE(),PRIVATE
LogoutHST            PROCEDURE(),BYTE,PROC,PRIVATE
OpenHST              PROCEDURE(),BYTE,PRIVATE
    END

sbkFileManager.DeleteRecord PROCEDURE(BYTE Query)
ReturnValue            BYTE,AUTO
  CODE
    FileName = SELF.File{PROP:Name}
    ThePosition = POSITION(SELF.File)
    TheRecord &= SELF.Buffer
    IF SELF.Incremented=False
      ReturnValue=AddHST(ACTION:Delete)
    ELSE
      ReturnValue=LEVEL:Benign
    END
    IF ReturnValue=LEVEL:Benign
      ReturnValue = PARENT.DeleteRecord(Query)
    END
    RETURN ReturnValue

sbkFileManager.Init  PROCEDURE(FILE File, ErrorClass E)
  CODE
    PARENT.Init(File,E)
    IF NOT SELF.HistoryFileName
      SELF.HistoryFileName='HST.TPS'
    END
    EC &= E
    FM &= SELF
    SELF.Incremented=False
    CALLBACK(File, SELF.MyFCB)

sbkFileManager.Insert PROCEDURE
ReturnValue            BYTE,AUTO
  CODE
    ReturnValue=PARENT.Insert()
    FileName = SELF.File{PROP:Name}
    ThePosition = POSITION(SELF.File)
    TheRecord &= SELF.Buffer
    IF ReturnValue=LEVEL:Benign
      ReturnValue=AddHST(ACTION:Insert)
      SELF.Incremented=False
    END
    RETURN ReturnValue

sbkFileManager.Kill  PROCEDURE
  CODE
    PARENT.Kill()
    IF NOT SELF.File &= NULL
      CALLBACK(SELF.File, SELF.MyFCB, True)
    END

sbkFileManager.PrimeAutoInc PROCEDURE
ReturnValue            BYTE,AUTO
  CODE
    ReturnValue=PARENT.PrimeAutoInc()
    IF ReturnValue=LEVEL:Benign
      SELF.Incremented=True
    END
    RETURN ReturnValue

sbkFileManager.TryInsert PROCEDURE
ReturnValue            BYTE,AUTO
  CODE
    ReturnValue=PARENT.TryInsert()
    FileName = SELF.File{PROP:Name}
    ThePosition = POSITION(SELF.File)
    TheRecord &= SELF.Buffer
    IF ReturnValue=LEVEL:Benign
      ReturnValue=AddHST(ACTION:Insert)
      SELF.Incremented=False
    END
    RETURN ReturnValue

sbkFileManager.TryPrimeAutoInc PROCEDURE
ReturnValue            BYTE,AUTO
  CODE
    ReturnValue=PARENT.TryPrimeAutoInc()
    IF ReturnValue=LEVEL:Benign
      SELF.Incremented=True
    END
    RETURN ReturnValue

sbkFileManager.TryUpdate PROCEDURE
ReturnValue            BYTE,AUTO
  CODE
    ReturnValue=PARENT.TryUpdate()
    FileName = SELF.File{PROP:Name}
    ThePosition = POSITION(SELF.File)
    TheRecord &= SELF.Buffer
    IF ReturnValue=LEVEL:Benign
      ReturnValue=AddHST(ACTION:Change)
    END
    RETURN ReturnValue

sbkFileManager.Update PROCEDURE
ReturnValue            BYTE,AUTO
  CODE
    ReturnValue=PARENT.Update()
    FileName = SELF.File{PROP:Name}
    ThePosition = POSITION(SELF.File)
    TheRecord &= SELF.Buffer
    IF ReturnValue=LEVEL:Benign
      ReturnValue=AddHST(ACTION:Change)
    END
    RETURN ReturnValue

AddHST               PROCEDURE(BYTE Action)
ReturnValue            BYTE,AUTO
  CODE
    ReturnValue = LogoutHST()
    IF ReturnValue=LEVEL:Benign
      CLEAR(HST)
      HST:Action = Action
      HST:FileName = FileName
      !HST:FileName = FM.File{PROP:Name} - why doesn't this work?
      HST:FileVersion = FM.FileVersion
      HST:Position = ThePosition
      !HST:Position = POSITION(FM.File) - why doesn't this work?
      HST:TheDate = TODAY()
      HST:TheTime = CLOCK()
      HST:TheUser = FM.TheUser

      !TheRecord &= SELF.File{PROP:Record} - why doesn't this work?
      HST:Buffer{PROP:Size} = 0
      HST:Buffer[0:SIZE(TheRecord)-1] = TheRecord

      HST:Generic = FM.GenericHistoryField
      ADD(HST)
      IF ERRORCODE()
        ReturnValue = EC.ThrowFile(Msg:AddFailed, NAME(HST))
      END
    END
    IF HSTlogged AND NOT LoggedWithCB
      IF ReturnValue=LEVEL:Benign
        COMMIT
      ELSE
        ROLLBACK
      END
      HSTlogged=False
      CloseHST()
    END
    RETURN ReturnValue

CloseHST             PROCEDURE()
  CODE
    CLOSE(HST)
    HSTopened=False

LogoutHST            PROCEDURE()
ReturnValue            BYTE(LEVEL:Benign)
  CODE
    IF NOT HSTlogged
      ReturnValue = OpenHST()
      IF ReturnValue=LEVEL:Benign
        HST{PROP:Logout}=1
        IF ERRORCODE() THEN EC.ThrowFile(Msg:LogoutFailed, NAME(HST)).
        IF NOT LoggedWithCB
          LOOP
            LOGOUT(1)
            IF NOT ERRORCODE()
              BREAK
            END
          END
        END
        HSTlogged=True
      END
    END
    RETURN ReturnValue

OpenHST              PROCEDURE()
ReturnValue            BYTE(LEVEL:Benign)
  CODE
    HST{PROP:Name}=FM.HistoryFileName
    IF NOT HSTopened
      CloseHST()
      IF NOT EXISTS(HST{PROP:Name})
        CREATE(HST)
        IF ERRORCODE()
          ReturnValue = EC.ThrowFile(Msg:CreateFailed, NAME(HST))
        END
      END
      IF ReturnValue=LEVEL:Benign
        OPEN(HST,42h)
        IF ERRORCODE()
          ReturnValue = EC.ThrowFile(Msg:OpenFailed, NAME(HST))
        END
      END
      IF ReturnValue = LEVEL:Benign
        HSTopened = True
      END
    END
    RETURN ReturnValue

sbkFileManager.MyFCB.FunctionCalled PROCEDURE(SIGNED opCode, *Params Parameters, *CSTRING ErrCode, *CSTRING ErrMsg)
  CODE
    IF opCode = DriverOp:LOGOUT AND NOT LoggedWithCB
      LoggedWithCB=True
      LogoutHST()
    END
    RETURN True

sbkFileManager.MyFCB.FunctionDone PROCEDURE(SIGNED opCode, *Params Parameters, *CSTRING ErrCode, *CSTRING ErrMsg)
  CODE
    IF (opCode = DriverOp:COMMIT OR opCode = DriverOp:ROLLBACK) AND LoggedWithCB
      HSTlogged=False
      LoggedWithCB=False
      CloseHST()
    END
    RETURN True
